import java.io.*;
import java.net.*;
import java.util.zip.Checksum;
import java.util.zip.CRC32;
import java.nio.ByteBuffer;

public class Client {

  private static final int PACKET_MAX_SIZE = 65536;
  private static final int HEADER_SIZE = 8;
  private static final int STARTING_CRC32_POSITION = 0;
  private static final int CRC32_SIZE = 8;
  private static final int DATA_MAX_SIZE = PACKET_MAX_SIZE - HEADER_SIZE;

  public static void main(final String[] args) {
    DatagramSocket clientSocket = null;
    InetAddress serverAddress = null;
    Checksum crc32Code = new CRC32();
    byte[] outputBuffer = new byte[PACKET_MAX_SIZE];

    // Inizializzazione
    try {
      clientSocket = new DatagramSocket();
      serverAddress = InetAddress.getByName("localhost");
    }catch (final SocketException e) {
      e.printStackTrace();
      System.exit(0);
    }catch (final UnknownHostException e2) {
      e2.printStackTrace();
      System.exit(0);
    }

    // Genero i dati da inviare
    byte[] dataBuffer;
    byte[] headerBuffer = new byte[CRC32_SIZE];
    long generatedCRC32Code;
    ByteBuffer headerByteBuffer = ByteBuffer.allocate(CRC32_SIZE);

    dataBuffer = new String("AABBAABBCICCIO").getBytes();
    crc32Code.reset();
    crc32Code.update(dataBuffer, 0, dataBuffer.length);
    generatedCRC32Code = crc32Code.getValue();
    headerByteBuffer.putLong(generatedCRC32Code);
    headerBuffer = headerByteBuffer.array();
    //headerByteBuffer.get(headerBuffer, 0, headerBuffer.length);

    // Costruisco il buffer finale da inviare.
    if(dataBuffer.length > DATA_MAX_SIZE) throw new IllegalStateException("Lunghezza dati eccessiva.");
    System.arraycopy(headerBuffer, 0, outputBuffer, STARTING_CRC32_POSITION, HEADER_SIZE);
    System.arraycopy(dataBuffer, 0, outputBuffer, HEADER_SIZE, dataBuffer.length);

    // Invio il buffer finale come datagramma UDP.
    final DatagramPacket sendData = new DatagramPacket(outputBuffer, HEADER_SIZE + dataBuffer.length, serverAddress, 8888);

    try {
      clientSocket.send(sendData);
    }catch (final IOException e) {
      e.printStackTrace();
    } finally {
      clientSocket.close();
    }
  }
}
