import java.io.*;
import java.net.*;
import java.util.zip.Checksum;
import java.util.zip.CRC32;
import java.nio.ByteBuffer;

/** L'obbiettivo di questa prova è quello di creare un protocollo applicativo
 *  semplice (codice di controllo + dati) e leggere dati provenienti dal client
 *  che siano più grandi della dimensione del buffer (+ di 65536).
 */
public class Server {

  private static final int PACKET_MAX_SIZE = 65536;
  private static final int HEADER_SIZE = 8;
  private static final int CRC32_SIZE = 8;
  private static final int STARTING_CRC32_POSITION = 0;
  private static final int DATA_MAX_SIZE = PACKET_MAX_SIZE - HEADER_SIZE;

  public static void main(final String[] args) {
	DatagramSocket serverSocket = null;

	try{
		serverSocket = new DatagramSocket(8888);
	}catch (final SocketException e) {
		e.printStackTrace();
		System.exit(0);
	}

  final byte[] receiveBuffer = new byte[PACKET_MAX_SIZE];
  int packetLength = 0;

  while(true) {
    final DatagramPacket receivedPacketD = new DatagramPacket(receiveBuffer, receiveBuffer.length);
	  try {
		  serverSocket.receive(receivedPacketD);
      packetLength = receivedPacketD.getLength();
	  } catch (final IOException e) {
		  e.printStackTrace();
		  System.exit(0);
	  }

    byte[] data = new byte[packetLength - HEADER_SIZE];
    long generatedCRC32Code;
    long receivedCRC32Code;

    // Prelevo il codice crc32 dal pacchetto ricevuto.
    ByteBuffer receivedCRC32ByteBuffer = ByteBuffer.allocate(CRC32_SIZE);
    receivedCRC32ByteBuffer.put(receiveBuffer, STARTING_CRC32_POSITION, CRC32_SIZE);
    receivedCRC32ByteBuffer.flip();
    receivedCRC32Code = receivedCRC32ByteBuffer.getLong();

    // Prelevo i dati contenuti nel datagramma
    System.arraycopy(receiveBuffer, HEADER_SIZE, data, 0, packetLength - HEADER_SIZE);
    final Checksum check = new CRC32();
    check.update(data, 0, data.length);
    generatedCRC32Code = check.getValue();

    if(generatedCRC32Code != receivedCRC32Code) {
      System.out.println("Codice CRC32 non valido. Datagramma corrotto.");
      continue;
    }

    String dataString = new String(data);
    System.out.println("Ricevuto: " + dataString);
  }
  }
}
